(alert mail "echo -n '[%state%] Problem with %function% %date% %params%' | mail -s '[%state%] alarm' mail@isp.net")
(alert sms  "/home/user/sms.sh '%date% %state% %function% %params% %hostname%")
(alert available-variables "REMINDER : %function% %params% %date% %hostname% %desc% %level% %os% %newline% %result% %state%")
(alert empty "")


;; this is a comment
; this is also a comment
(=> mail file-updated :path "/tmp/reed-alert" :limit 100)
(=> mail disk-usage :path "/" :limit 90)

(=> mail service :name "dovecot")
(=> mail service :name "httpd")
(=> sms service  :name "smtpd")
(=> mail number-of-processes :limit 100)

(=> mail ping :host "bitreich.org" :desc "Ping Bitreich")
(=> mail ping :host "openbsd.org"  :desc "Ping OpenBSD.org")
(=> mail ssl-expiration :host "freenode.net" :seconds (* 7 24 60 60) :desc "SSL IRC Freenode")
(=> mail write-to-file :path "/tmp/reed-alert")
